@@ -17,18 +17,22 @@
 # main is only used when canto-fetch is called from the command line.
 # run is used internally by canto when it needs to invoke an update.
 
+import feedparser_builtin
+
+feedparser = feedparser_builtin
+
 from const import VERSION_TUPLE, GIT_SHA
 from cfg.base import get_cfg
 import utility
 import args
 
 from threading import Thread
-import feedparser
 import traceback
 import commands
 import urlparse
 import urllib2
 import cPickle
+import locale
 import socket
 import signal
 import fcntl
@@ -39,8 +43,8 @@ import os
 def main(enc):
     conf_dir, log_file, conf_file, feed_dir, script_dir, optlist =\
         args.parse_common_args(enc,
-            "hvVfdbi:", ["help","version","verbose","force","daemon",\
-                    "background", "interval="], "canto-fetch")
+            "hvVfdbi:s", ["help","version","verbose","force","daemon",\
+                    "background", "interval=", "sysfp"], "canto-fetch")
 
     try :
         cfg = get_cfg(conf_file, log_file, feed_dir, script_dir)
@@ -84,6 +88,14 @@ def main(enc):
                 cfg.log("%s isn't a valid interval" % arg)
             else:
                 cfg.log("interval = %d seconds" % updateInterval)
+        if opt in ["-s","--sysfp"]:
+            log_func("Using system feedparser")
+            global feedparser
+            try:
+                import feedparser as feedparser_system
+                feedparser = feedparser_system
+            except:
+                log_func("Import failed. Falling back on builtin.")
         if opt in ["-V","--verbose"]:
             verbose = True
         elif opt in ["-f","--force"]:
@@ -313,8 +325,9 @@ class FetchThread(Thread):
             # case we either won't get data or can't trust the data, so
             # just skip processing this feed.
 
+            enc = locale.getpreferredencoding()
             self.log_func("Exception trying to get feed %s : %s" % \
-                    (self.fd.tags[0], sys.exc_info()[1]))
+                    (self.fd.URL.encode(enc, "ignore"), sys.exc_info()[1]))
 
             return
 
