@@ -24,48 +24,44 @@
 # Python's clock (see: Haskell, Erlang).
 # --END SIDE RANT--
 #
-# Multiprocessing in canto is not simple. The ProcessManager class forks a
-# process with two pipes, one directed to the process and another directed from
-# the process (the typical pipe setup). The interface process puts a work object
-# in the pipe, the worker process does the work and puts the result in the
-# outgoing pipe. The work tuples looke like this:
+# Processing in canto is not simple. The ProcessManager class forks a process
+# with two pipes, one directed to the process and another directed from the
+# process (the typical pipe setup). The interface process puts a work object in
+# the pipe, the worker process does the work and puts the result in the outgoing
+# pipe. The work tuples looke like this:
 #
 #       (action, arguments...)
 #
-# There are a number of actions:
-#   (PROC_UPDATE, URL, old items)
-#      performs on disk update only, this is used early in init when we're
-#      trying to rectify tags from ondisk content
+# There are a number of actions: (PROC_UPDATE, URL, old items) performs on disk
+# update only, this is used early in init when we're trying to rectify tags from
+# ondisk content
 #
-#   (PROC_GETTAGS, )
-#      requests that the process return the rectified tags (i.e. collisions
-#      resolved)
+#   (PROC_GETTAGS, ) requests that the process return the rectified tags (i.e.
+#       collisions resolved)
 #
 #   (PROC_FILTER / PROCESS_BOTH , URL, old items, global filter index,
-#    [tag_info], refilter)
-#      performs the filtering/sorting (in addition to update for BOTH) this is
-#      the most common full update. PROC_FILTER is only used after PROC_UPDATE
-#      early on. [tag_info] is a list of one tuple per tag containing:
+#       [tag_info], refilter) performs the filtering/sorting (in addition to
+#       update for BOTH) this is the most common full update. PROC_FILTER is
+#       only used after PROC_UPDATE early on. [tag_info] is a list of one tuple
+#       per tag containing:
 #
 #         (tag string, tag filter index, tag sort index)
 #
-#   (PROC_FLUSH, )
-#      This essentially serves as a marker in the pipe that's returned verbatim
-#      when the worker thread receives it. In practice, the ProcessHandler's
-#      flush() call puts it into the pipe and discards any output until it gets
-#      it back. This is used when the items in the pipe are no longer accurate
-#      (i.e. the filter/sort/tag settings have changed.
+#   (PROC_FLUSH, ) This essentially serves as a marker in the pipe that's
+#       returned verbatim when the worker thread receives it. In practice, the
+#       ProcessHandler's flush() call puts it into the pipe and discards any
+#       output until it gets it back. This is used when the items in the pipe
+#       are no longer accurate (i.e. the filter/sort/tag settings have changed.
 #
-#   (PROC_SYNC, URL, old items)
-#      This syncs the state to disk. Typically the state is saved on update, but
-#      on exiting the program, it needs to be explicitly synced to disk so that
-#      any state changes made between the last disk update are saved.
+#   (PROC_SYNC, URL, old items) This syncs the state to disk. Typically the
+#       state is saved on update, but on exiting the program, it needs to be
+#       explicitly synced to disk so that any state changes made between the
+#       last disk update are saved.
 #
-#   (PROC_KILL, )
-#      Kills the process. In implementation, it's just PROC_FLUSH with an added
-#      clause so it terminates after returning the same tuple. After that tuple
-#      if received back, it's guaranteed that the process was safely exited and
-#      it can be assumed that the pipes are no longer active.
+#   (PROC_KILL, ) Kills the process. In implementation, it's just PROC_FLUSH
+#       with an added clause so it terminates after returning the same tuple.
+#       After that tuple if received back, it's guaranteed that the process was
+#       safely exited and it can be assumed that the pipes are no longer active.
 
 # Most of the return tuples are self-explanatory. The most common return from
 # PROC_BOTH looks like this:
@@ -106,39 +102,85 @@
 
 from const import *
 
+from threading import Thread, Lock
+from cPickle import dumps, loads
+import select
 import signal
 import time
 import sys
 import os
 
-try:
-    from multiprocessing import Process, Queue
-except Exception, e:
-    try:
-        from processing import Process, Queue
-    except:
-        print "Canto 0.7.x requires the python-processing module"+\
-              " to run on Python 2.5"
-        sys.exit(-1)
+class Queue():
+    def __init__(self):
+        self.recvpipe, self.sendpipe = os.pipe()
+
+        self.poll = select.poll()
+        self.poll.register(self.recvpipe, select.POLLIN)
+
+        self.objlist = []
+        self.objlock = Lock()
+
+        self.thread = None
+        self.alive = True
+
+    def get(self, block=True, timeout=None):
+        ready = self.poll.poll(timeout)
+        for fd, event in ready:
+            l = os.read(self.recvpipe, 1)
+            while l[-1] != " ":
+                l += os.read(self.recvpipe, 1)
+            l = int(l)
+
+            s = os.read(self.recvpipe, l)
+            return loads(s)
+        raise Exception
+
+    def feed_pipe(self):
+        while self.alive:
+            if not len(self.objlist):
+                time.sleep(0.1)
+                continue
+            self.objlock.acquire()
+            obj = self.objlist[0]
+            self.objlist = self.objlist[1:]
+            self.objlock.release()
+
+            s = dumps(obj)
+            l = len(s)
+            os.write(self.sendpipe, "%d %s" % (l, s))
+
+    def put(self, obj):
+        if not self.thread:
+            self.thread = Thread(target = self.feed_pipe)
+            self.thread.start()
+        self.objlock.acquire()
+        self.objlist.append(obj)
+        self.objlock.release()
+
+    def close(self):
+        if self.thread:
+            while len(self.objlist): pass
+            self.alive = False
+            self.thread.join()
+
+        os.close(self.recvpipe)
+        os.close(self.sendpipe)
 
 class ProcessHandler():
     def __init__(self, cfg):
+        self.persist = True
         self.cfg = cfg
-        self.start_process(cfg)
+        self.start_process(cfg, True)
 
-    def start_process(self, cfg):
+    def start_process(self, cfg, persist=False):
+        self.persist = persist
         self.updated = Queue()
         self.update = Queue()
-        self.process = \
-                Process(target = self.run,
-                        args = (self.update, self.updated,
-                            cfg.all_filters, cfg.all_sorts, cfg.feeds))
-
-	try:
-	    self.process.start()
-        except:
-            return False
-        return True
+        self.pid = os.fork()
+        if not self.pid:
+            self.run(self.update, self.updated, cfg.all_filters,
+                    cfg.all_sorts, cfg.feeds)
+        return self.pid
 
     def run(self, update, updated, all_filters, all_sorts, feeds):
         def scan_tags(feeds):
@@ -182,11 +224,11 @@ class ProcessHandler():
         # the worker, so that's the only one that's truly necessary
         # AFAICT, but chalk the rest of the ignores up to paranoia.
 
-        signal.signal(signal.SIGCHLD, signal.SIG_IGN)
-        signal.signal(signal.SIGWINCH, signal.SIG_IGN)
-        signal.signal(signal.SIGALRM, signal.SIG_IGN)
-        signal.signal(signal.SIGINT, signal.SIG_IGN)
-        signal.signal(signal.SIGUSR1, signal.SIG_IGN)
+        signal.signal(signal.SIGCHLD, signal.SIG_DFL)
+        signal.signal(signal.SIGWINCH, signal.SIG_DFL)
+        signal.signal(signal.SIGALRM, signal.SIG_DFL)
+        signal.signal(signal.SIGINT, signal.SIG_DFL)
+        signal.signal(signal.SIGUSR1, signal.SIG_DFL)
 
         while True:
             while True:
@@ -284,7 +326,7 @@ class ProcessHandler():
                 for i, (t, tf, ts) in enumerate(taginfo):
                     sort = all_sorts[ts]
                     if not sort:
-                        break
+                        continue
                     if ndiff[i]:
                         ndiff[i].sort(sort)
                     if odiff[i]:
@@ -315,9 +357,15 @@ class ProcessHandler():
                 del feed[:]
 
     def send(self, obj):
+        if not self.pid:
+            self.start_process(self.cfg)
         return self.update.put(obj)
 
-    def recv(self, block=True, timeout=None):
+    # recv_raw won't attempt to kill the process if no more work is queued.
+    # Its intended for use when syncing / killing when a tuple is floated
+    # through the process and no feeds are queued.
+
+    def recv_raw(self, block=True, timeout=None):
         r = None
         try:
             r = self.updated.get(block, timeout)
@@ -325,10 +373,24 @@ class ProcessHandler():
             pass
         return r
 
+    def recv(self, block=True, timeout=None):
+        r = self.recv_raw(block, timeout)
+
+        # If no more feeds are queued and we're not persistent (used very early), 
+        # kill the slave process in addition to returning the received value.
+        if self.persist:
+            return r
+        for f in self.cfg.feeds:
+            if f.qd:
+                return r
+
+        self.kill_process()
+        return r
+
     def send_and_wait(self, symbol):
         self.send((symbol, ))
         while True:
-            got = self.recv()
+            got = self.recv_raw()
             if got == (symbol, ):
                 return
 
@@ -339,34 +401,18 @@ class ProcessHandler():
             f.qd = False
 
     def kill_process(self):
-        # Fuck around with the signals because multiprocessing
-        # chokes unless SIGCHLD == SIG_DFL.
-
-        # This seems dangerous, but it relies on the fact that kill_process
-        # should only be called in a context in which Canto's not waiting for a
-        # text based forked process (see self.chld in Main for more).
-
-        sig = signal.getsignal(signal.SIGCHLD)
-        signal.signal(signal.SIGCHLD, signal.SIG_DFL)
-
-        self.send_and_wait(PROC_KILL)
-        self.update.close()
-        self.updated.close()
-
-        # OSX seems to not like the process join.
-        # We shouldn't care, since it should be dead
-        # by now anyway.
-        try:
-            self.process.join()
-        except:
-            pass
-
-        signal.signal(signal.SIGCHLD, sig)
+        if self.pid:
+            self.send_and_wait(PROC_KILL)
+            self.update.close()
+            self.updated.close()
+            self.pid = 0
 
     def flush(self):
-        self.send_and_wait(PROC_FLUSH)
+        if self.pid:
+            self.send_and_wait(PROC_FLUSH)
 
     def sync(self):
         for f in self.cfg.feeds:
             self.send((PROC_SYNC, f.URL, f[:]))
-            self.recv()
+        for f in self.cfg.feeds:
+            self.recv_raw()
