@@ -96,9 +96,7 @@ class Reader(BaseGui):
     def toggle_show_links(self):
         self.show_links = not self.show_links
         self.refresh()
-
-        if not self.show_links:
-            return REDRAW_ALL
+        return REDRAW_ALL
 
     def scroll_down(self):
         if self.more > 0 :
